#include "drawwidget.h"
#include <QKeyEvent>
#include <QMouseEvent>
#include <scene.h>
#include <pipeline.h>
#include <shadersource.h>
#include <framebuffer.h>
#include <resource.h>
#include <glm/ext.hpp>
#include "map.h"
#include "pixelpipeline.h"
#include <QCoreApplication>

DrawWidget::DrawWidget(QWidget *parent) :
    QOpenGLWidget(parent),
    m_Qt_fbo(NULL)
{
    connect(&openglRefreshTimer, SIGNAL(timeout()), this, SLOT(repaint()));
    openglRefreshTimer.start(16);
    m_map = new Map(200, 100);
}

void DrawWidget::initializeGL()
{
    renderer.initGL(width(), height());

    m_pipeline = new PixelPipeline(200, 100);
    m_map->setPipeline(m_pipeline);

    m_pipeline->pushChange(glm::vec2(50, 50), glm::vec3(0, 1, 0));
    m_pipeline->pushChange(glm::vec2(150, 50), glm::vec3(0, 0, 1));
    m_pipeline->updateChanges();

    renderer.setScene(m_map);
}

void DrawWidget::paintGL()
{
    renderer.renderGL();
}

void DrawWidget::resizeGL(int w, int h)
{
    renderer.resizeGL(w, h);
    if(m_Qt_fbo != NULL && m_Qt_fbo != FrameBuffer::screen)
        delete(m_Qt_fbo);
    m_Qt_fbo = new FrameBuffer(defaultFramebufferObject());
    m_pipeline->setTargetFBO(m_Qt_fbo);
}

void DrawWidget::mouseMoveEvent(QMouseEvent *event)
{
    if(grabbedMouse)
        m_pipeline->cameraMove(event->globalX() - lastMousePos.x(), event->globalY() - lastMousePos.y());
    lastMousePos = event->globalPos();
}

void DrawWidget::mousePressEvent(QMouseEvent* event)
{
    switch(event->button())
    {
        case Qt::LeftButton :
            grabbedMouse = true;
            break;
        default:
            break;
    }
    lastMousePos = event->globalPos();
}

void DrawWidget::mouseDoubleClickEvent(QMouseEvent * event)
{
    
}

void DrawWidget::mouseReleaseEvent(QMouseEvent* event)
{
    switch(event->button())
    {
        case Qt::LeftButton :
            grabbedMouse = false;
            break;
        default:
            break;
    }
}

void DrawWidget::wheelEvent(QWheelEvent *event)
{
    m_pipeline->cameraZoom(event->delta());
}
