#include "libwidget.h"
#include "ui_libwidget.h"

#include <QDir>
#include <QLibrary>

#ifdef Q_OS_WIN
#define LIB_SUFFIX "dll"
#else
#define LIB_SUFFIX "so"
#endif

typedef void (*ThinkFunction)(void);

LibWidget::LibWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::LibWidget)
{
    ui->setupUi(this);
    refreshLibs();
    connect(ui->pushButton, SIGNAL(pressed()), this, SLOT(refreshLibs()));
}

LibWidget::~LibWidget()
{
    delete ui;
}

void LibWidget::refreshLibs()
{
    while(ui->listWidget->count() > 0)
        ui->listWidget->takeItem(0);
    QDir teamDir(QCoreApplication::applicationDirPath());
    if(teamDir.cd("../teams"))
    {
        for(const QFileInfo &info : teamDir.entryInfoList())
        {
            if(info.suffix().compare(LIB_SUFFIX) == 0)
            {
                QLibrary lib(info.absoluteFilePath());
                if(lib.load())
                {
                    ThinkFunction func = lib.resolve("think");
                    if(func)
                    {
                        ui->listWidget->addItem(info.baseName());
                    }
                }
            }
        }
    }
    else
        emit sendError(QString("ERROR : can't open the teams folder.\n"), 5000);
}
